@@ -92,16 +92,21 @@ void mp_task(void *pvParameter) {
     usb_init();
     #elif CONFIG_ESP_CONSOLE_USB_SERIAL_JTAG
     usb_serial_jtag_init();
-    #else
-    uart_init();
+    #endif
+    #if MICROPY_HW_ENABLE_UART_REPL
+    uart_stdout_init();
     #endif
     machine_init();
 
-    // TODO: CONFIG_SPIRAM_SUPPORT is for 3.3 compatibility, remove after move to 4.0.
-    #if CONFIG_ESP32_SPIRAM_SUPPORT || CONFIG_SPIRAM_SUPPORT
-    // Try to use the entire external SPIRAM directly for the heap
     size_t mp_task_heap_size;
-    void *mp_task_heap = (void *)SOC_EXTRAM_DATA_LOW;
+    void *mp_task_heap = NULL;
+
+    #if CONFIG_SPIRAM_USE_MALLOC
+    // SPIRAM is issued using MALLOC, fallback to normal allocation rules
+    mp_task_heap = NULL;
+    #elif CONFIG_ESP32_SPIRAM_SUPPORT
+    // Try to use the entire external SPIRAM directly for the heap
+    mp_task_heap = (void *)SOC_EXTRAM_DATA_LOW;
     switch (esp_spiram_get_chip_size()) {
         case ESP_SPIRAM_SIZE_16MBITS:
             mp_task_heap_size = 2 * 1024 * 1024;
@@ -112,27 +117,31 @@ void mp_task(void *pvParameter) {
             break;
         default:
             // No SPIRAM, fallback to normal allocation
-            mp_task_heap_size = heap_caps_get_largest_free_block(MALLOC_CAP_8BIT);
-            mp_task_heap = malloc(mp_task_heap_size);
+            mp_task_heap = NULL;
             break;
     }
     #elif CONFIG_ESP32S2_SPIRAM_SUPPORT || CONFIG_ESP32S3_SPIRAM_SUPPORT
     // Try to use the entire external SPIRAM directly for the heap
-    size_t mp_task_heap_size;
     size_t esp_spiram_size = esp_spiram_get_size();
-    void *mp_task_heap = (void *)SOC_EXTRAM_DATA_HIGH - esp_spiram_size;
     if (esp_spiram_size > 0) {
+        mp_task_heap = (void *)SOC_EXTRAM_DATA_HIGH - esp_spiram_size;
         mp_task_heap_size = esp_spiram_size;
-    } else {
-        // No SPIRAM, fallback to normal allocation
-        mp_task_heap_size = heap_caps_get_largest_free_block(MALLOC_CAP_8BIT);
+    }
+    #endif
+
+    if (mp_task_heap == NULL) {
+        // Allocate the uPy heap using malloc and get the largest available region,
+        // limiting to 1/2 total available memory to leave memory for the OS.
+        #if ESP_IDF_VERSION >= ESP_IDF_VERSION_VAL(4, 1, 0)
+        size_t heap_total = heap_caps_get_total_size(MALLOC_CAP_8BIT);
+        #else
+        multi_heap_info_t info;
+        heap_caps_get_info(&info, MALLOC_CAP_8BIT);
+        size_t heap_total = info.total_free_bytes + info.total_allocated_bytes;
+        #endif
+        mp_task_heap_size = MIN(heap_caps_get_largest_free_block(MALLOC_CAP_8BIT), heap_total / 2);
         mp_task_heap = malloc(mp_task_heap_size);
     }
-    #else
-    // Allocate the uPy heap using malloc and get the largest available region
-    size_t mp_task_heap_size = heap_caps_get_largest_free_block(MALLOC_CAP_8BIT);
-    void *mp_task_heap = malloc(mp_task_heap_size);
-    #endif
 
 soft_reset:
     // initialise the stack pointer for the main thread
@@ -143,6 +152,8 @@ soft_reset:
     mp_obj_list_append(mp_sys_path, MP_OBJ_NEW_QSTR(MP_QSTR__slash_lib));
     readline_init0();
 
+    MP_STATE_PORT(native_code_pointers) = MP_OBJ_NULL;
+
     // initialise peripherals
     machine_pins_init();
     #if MICROPY_PY_MACHINE_I2S
@@ -185,6 +196,16 @@ soft_reset_exit:
     mp_thread_deinit();
     #endif
 
+    // Free any native code pointers that point to iRAM.
+    if (MP_STATE_PORT(native_code_pointers) != MP_OBJ_NULL) {
+        size_t len;
+        mp_obj_t *items;
+        mp_obj_list_get(MP_STATE_PORT(native_code_pointers), &len, &items);
+        for (size_t i = 0; i < len; ++i) {
+            heap_caps_free(MP_OBJ_TO_PTR(items[i]));
+        }
+    }
+
     gc_sweep_all();
 
     mp_hal_stdout_tx_str("MPY: soft reboot\r\n");
@@ -234,6 +255,10 @@ void *esp_native_code_commit(void *buf, size_t len, void *reloc) {
     if (p == NULL) {
         m_malloc_fail(len);
     }
+    if (MP_STATE_PORT(native_code_pointers) == MP_OBJ_NULL) {
+        MP_STATE_PORT(native_code_pointers) = mp_obj_new_list(0, NULL);
+    }
+    mp_obj_list_append(MP_STATE_PORT(native_code_pointers), MP_OBJ_TO_PTR(p));
     if (reloc) {
         mp_native_relocate(reloc, buf, (uintptr_t)p);
     }
