@@ -35,22 +35,22 @@ sys.path.append(os.path.dirname(__file__) + "/../py")
 import makeqstrdata as qstrutil
 
 # MicroPython constants
-MPY_VERSION = 5
+MPY_VERSION = 6
+MP_CODE_BYTECODE = 2
+MP_CODE_NATIVE_VIPER = 4
 MP_NATIVE_ARCH_X86 = 1
 MP_NATIVE_ARCH_X64 = 2
+MP_NATIVE_ARCH_ARMV6M = 4
 MP_NATIVE_ARCH_ARMV7M = 5
 MP_NATIVE_ARCH_ARMV7EMSP = 7
 MP_NATIVE_ARCH_ARMV7EMDP = 8
 MP_NATIVE_ARCH_XTENSA = 9
 MP_NATIVE_ARCH_XTENSAWIN = 10
-MP_CODE_BYTECODE = 2
-MP_CODE_NATIVE_VIPER = 4
+MP_PERSISTENT_OBJ_STR = 5
 MP_SCOPE_FLAG_VIPERRELOC = 0x10
 MP_SCOPE_FLAG_VIPERRODATA = 0x20
 MP_SCOPE_FLAG_VIPERBSS = 0x40
-MICROPY_PY_BUILTINS_STR_UNICODE = 2
 MP_SMALL_INT_BITS = 31
-QSTR_WINDOW_SIZE = 32
 
 # ELF constants
 R_386_32 = 1
@@ -75,6 +75,7 @@ R_ARM_THM_JUMP24 = 30
 R_X86_64_GOTPCREL = 9
 R_X86_64_REX_GOTPCRELX = 42
 R_386_GOT32X = 43
+R_XTENSA_PDIFF32 = 59
 
 ################################################################################
 # Architecture configuration
@@ -84,7 +85,14 @@ def asm_jump_x86(entry):
     return struct.pack("<BI", 0xE9, entry - 5)
 
 
-def asm_jump_arm(entry):
+def asm_jump_thumb(entry):
+    # Only signed values that fit in 12 bits are supported
+    b_off = entry - 4
+    assert b_off >> 11 == 0 or b_off >> 11 == -1, b_off
+    return struct.pack("<H", 0xE000 | (b_off >> 1 & 0x07FF))
+
+
+def asm_jump_thumb2(entry):
     b_off = entry - 4
     if b_off >> 11 == 0 or b_off >> 11 == -1:
         # Signed value fits in 12 bits
@@ -104,72 +112,73 @@ def asm_jump_xtensa(entry):
 
 
 class ArchData:
-    def __init__(self, name, mpy_feature, qstr_entry_size, word_size, arch_got, asm_jump):
+    def __init__(self, name, mpy_feature, word_size, arch_got, asm_jump, *, separate_rodata=False):
         self.name = name
         self.mpy_feature = mpy_feature
-        self.qstr_entry_size = qstr_entry_size
+        self.qstr_entry_size = 2
         self.word_size = word_size
         self.arch_got = arch_got
         self.asm_jump = asm_jump
-        self.separate_rodata = name == "EM_XTENSA" and qstr_entry_size == 4
+        self.separate_rodata = separate_rodata
 
 
 ARCH_DATA = {
     "x86": ArchData(
         "EM_386",
-        MP_NATIVE_ARCH_X86 << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_X86 << 2,
         4,
         (R_386_PC32, R_386_GOT32, R_386_GOT32X),
         asm_jump_x86,
     ),
     "x64": ArchData(
         "EM_X86_64",
-        MP_NATIVE_ARCH_X64 << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_X64 << 2,
         8,
         (R_X86_64_GOTPCREL, R_X86_64_REX_GOTPCRELX),
         asm_jump_x86,
     ),
-    "armv7m": ArchData(
+    "armv6m": ArchData(
         "EM_ARM",
-        MP_NATIVE_ARCH_ARMV7M << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_ARMV6M << 2,
         4,
         (R_ARM_GOT_BREL,),
-        asm_jump_arm,
+        asm_jump_thumb,
+    ),
+    "armv7m": ArchData(
+        "EM_ARM",
+        MP_NATIVE_ARCH_ARMV7M << 2,
+        4,
+        (R_ARM_GOT_BREL,),
+        asm_jump_thumb2,
     ),
     "armv7emsp": ArchData(
         "EM_ARM",
-        MP_NATIVE_ARCH_ARMV7EMSP << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_ARMV7EMSP << 2,
         4,
         (R_ARM_GOT_BREL,),
-        asm_jump_arm,
+        asm_jump_thumb2,
     ),
     "armv7emdp": ArchData(
         "EM_ARM",
-        MP_NATIVE_ARCH_ARMV7EMDP << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_ARMV7EMDP << 2,
         4,
         (R_ARM_GOT_BREL,),
-        asm_jump_arm,
+        asm_jump_thumb2,
     ),
     "xtensa": ArchData(
         "EM_XTENSA",
-        MP_NATIVE_ARCH_XTENSA << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        2,
+        MP_NATIVE_ARCH_XTENSA << 2,
         4,
         (R_XTENSA_32, R_XTENSA_PLT),
         asm_jump_xtensa,
     ),
     "xtensawin": ArchData(
         "EM_XTENSA",
-        MP_NATIVE_ARCH_XTENSAWIN << 2 | MICROPY_PY_BUILTINS_STR_UNICODE,
-        4,
+        MP_NATIVE_ARCH_XTENSAWIN << 2,
         4,
         (R_XTENSA_32, R_XTENSA_PLT),
         asm_jump_xtensa,
+        separate_rodata=True,
     ),
 }
 
@@ -421,10 +430,12 @@ def populate_got(env):
 
     # Create a relocation for each GOT entry
     for got_entry in got_list:
-        if got_entry.name == "mp_fun_table":
-            dest = "mp_fun_table"
+        if got_entry.name in ("mp_native_qstr_table", "mp_native_obj_table", "mp_fun_table"):
+            dest = got_entry.name
         elif got_entry.name.startswith("mp_fun_table+0x"):
             dest = int(got_entry.name.split("+")[1], 16) // env.arch.word_size
+        elif got_entry.sec_name == ".external.mp_fun_table":
+            dest = got_entry.sym.mp_fun_table_offset
         elif got_entry.sec_name.startswith(".text"):
             dest = ".text"
         elif got_entry.sec_name.startswith(".rodata"):
@@ -564,9 +575,9 @@ def do_relocation_text(env, text_addr, r):
         reloc = addr - r_offset
         reloc_type = "xtensa_l32r"
 
-    elif env.arch.name == "EM_XTENSA" and r_info_type == R_XTENSA_DIFF32:
+    elif env.arch.name == "EM_XTENSA" and r_info_type in (R_XTENSA_DIFF32, R_XTENSA_PDIFF32):
         if s.section.name.startswith(".text"):
-            # it looks like R_XTENSA_DIFF32 into .text is already correctly relocated
+            # it looks like R_XTENSA_[P]DIFF32 into .text is already correctly relocated
             return
         assert 0
 
@@ -740,19 +751,19 @@ def link_objects(env, native_qstr_vals_len, native_qstr_objs_len):
         env.lit_section = Section("LIT", bytearray(lit_size), env.arch.word_size)
         env.sections.insert(1, env.lit_section)
 
-    # Create section to contain mp_native_qstr_val_table
-    env.qstr_val_section = Section(
-        ".text.QSTR_VAL",
+    # Create section to contain mp_native_qstr_table
+    env.qstr_table_section = Section(
+        ".external.qstr_table",
         bytearray(native_qstr_vals_len * env.arch.qstr_entry_size),
         env.arch.qstr_entry_size,
     )
-    env.sections.append(env.qstr_val_section)
 
-    # Create section to contain mp_native_qstr_obj_table
-    env.qstr_obj_section = Section(
-        ".text.QSTR_OBJ", bytearray(native_qstr_objs_len * env.arch.word_size), env.arch.word_size
+    # Create section to contain mp_native_obj_table
+    env.obj_table_section = Section(
+        ".external.obj_table",
+        bytearray(native_qstr_objs_len * env.arch.word_size),
+        env.arch.word_size,
     )
-    env.sections.append(env.qstr_obj_section)
 
     # Resolve unknown symbols
     mp_fun_table_sec = Section(".external.mp_fun_table", b"", 0)
@@ -782,10 +793,10 @@ def link_objects(env, native_qstr_vals_len, native_qstr_objs_len):
             pass
         elif sym.name == "mp_fun_table":
             sym.section = Section(".external", b"", 0)
-        elif sym.name == "mp_native_qstr_val_table":
-            sym.section = env.qstr_val_section
-        elif sym.name == "mp_native_qstr_obj_table":
-            sym.section = env.qstr_obj_section
+        elif sym.name == "mp_native_qstr_table":
+            sym.section = env.qstr_table_section
+        elif sym.name == "mp_native_obj_table":
+            sym.section = env.obj_table_section
         elif sym.name in env.known_syms:
             sym.resolved = env.known_syms[sym.name]
         else:
@@ -860,11 +871,12 @@ class MPYOutput:
 
     def write_qstr(self, s):
         if s in qstrutil.static_qstr_list:
-            self.write_bytes(bytes([0, qstrutil.static_qstr_list.index(s) + 1]))
+            self.write_uint((qstrutil.static_qstr_list.index(s) + 1) << 1 | 1)
         else:
             s = bytes(s, "ascii")
             self.write_uint(len(s) << 1)
             self.write_bytes(s)
+            self.write_bytes(b"\x00")
 
     def write_reloc(self, base, offset, dest, n):
         need_offset = not (base == self.prev_base and offset == self.prev_offset + 1)
@@ -905,29 +917,31 @@ def build_mpy(env, entry_offset, fmpy, native_qstr_vals, native_qstr_objs):
     out.open(fmpy)
 
     # MPY: header
-    out.write_bytes(
-        bytearray(
-            [ord("M"), MPY_VERSION, env.arch.mpy_feature, MP_SMALL_INT_BITS, QSTR_WINDOW_SIZE]
-        )
-    )
+    out.write_bytes(bytearray([ord("M"), MPY_VERSION, env.arch.mpy_feature, MP_SMALL_INT_BITS]))
+
+    # MPY: n_qstr
+    out.write_uint(1 + len(native_qstr_vals))
+
+    # MPY: n_obj
+    out.write_uint(len(native_qstr_objs))
+
+    # MPY: qstr table
+    out.write_qstr(fmpy)  # filename
+    for q in native_qstr_vals:
+        out.write_qstr(q)
+
+    # MPY: object table
+    for q in native_qstr_objs:
+        out.write_bytes(bytearray([MP_PERSISTENT_OBJ_STR]))
+        out.write_uint(len(q))
+        out.write_bytes(bytes(q, "utf8") + b"\x00")
 
     # MPY: kind/len
-    out.write_uint(len(env.full_text) << 2 | (MP_CODE_NATIVE_VIPER - MP_CODE_BYTECODE))
+    out.write_uint(len(env.full_text) << 3 | (MP_CODE_NATIVE_VIPER - MP_CODE_BYTECODE))
 
     # MPY: machine code
     out.write_bytes(env.full_text)
 
-    # MPY: n_qstr_link (assumes little endian)
-    out.write_uint(len(native_qstr_vals) + len(native_qstr_objs))
-    for q in range(len(native_qstr_vals)):
-        off = env.qstr_val_section.addr + q * env.arch.qstr_entry_size
-        out.write_uint(off << 2)
-        out.write_qstr(native_qstr_vals[q])
-    for q in range(len(native_qstr_objs)):
-        off = env.qstr_obj_section.addr + q * env.arch.word_size
-        out.write_uint(off << 2 | 3)
-        out.write_qstr(native_qstr_objs[q])
-
     # MPY: scope_flags
     scope_flags = MP_SCOPE_FLAG_VIPERRELOC
     if len(env.full_rodata):
@@ -936,20 +950,15 @@ def build_mpy(env, entry_offset, fmpy, native_qstr_vals, native_qstr_objs):
         scope_flags |= MP_SCOPE_FLAG_VIPERBSS
     out.write_uint(scope_flags)
 
-    # MPY: n_obj
-    out.write_uint(0)
-
-    # MPY: n_raw_code
-    out.write_uint(0)
-
-    # MPY: rodata and/or bss
+    # MPY: bss and/or rodata
     if len(env.full_rodata):
         rodata_const_table_idx = 1
         out.write_uint(len(env.full_rodata))
-        out.write_bytes(env.full_rodata)
     if len(env.full_bss):
-        bss_const_table_idx = bool(env.full_rodata) + 1
+        bss_const_table_idx = 2
         out.write_uint(len(env.full_bss))
+    if len(env.full_rodata):
+        out.write_bytes(env.full_rodata)
 
     # MPY: relocation information
     prev_kind = None
@@ -963,10 +972,14 @@ def build_mpy(env, entry_offset, fmpy, native_qstr_vals, native_qstr_objs):
                 kind = 0
         elif isinstance(kind, str) and kind.startswith(".bss"):
             kind = bss_const_table_idx
-        elif kind == "mp_fun_table":
+        elif kind == "mp_native_qstr_table":
             kind = 6
+        elif kind == "mp_native_obj_table":
+            kind = 7
+        elif kind == "mp_fun_table":
+            kind = 8
         else:
-            kind = 7 + kind
+            kind = 9 + kind
         assert addr % env.arch.word_size == 0, addr
         offset = addr // env.arch.word_size
         if kind == prev_kind and base == prev_base and offset == prev_offset + 1:
@@ -1008,18 +1021,14 @@ def do_preprocess(args):
         for i, q in enumerate(static_qstrs):
             print("#define %s (%u)" % (q, i + 1), file=f)
         for i, q in enumerate(sorted(qstr_vals)):
-            print("#define %s (mp_native_qstr_val_table[%d])" % (q, i), file=f)
+            print("#define %s (mp_native_qstr_table[%d])" % (q, i + 1), file=f)
         for i, q in enumerate(sorted(qstr_objs)):
             print(
-                "#define MP_OBJ_NEW_QSTR_%s ((mp_obj_t)mp_native_qstr_obj_table[%d])" % (q, i),
+                "#define MP_OBJ_NEW_QSTR_%s ((mp_obj_t)mp_native_obj_table[%d])" % (q, i),
                 file=f,
             )
-        if args.arch == "xtensawin":
-            qstr_type = "uint32_t"  # esp32 can only read 32-bit values from IRAM
-        else:
-            qstr_type = "uint16_t"
-        print("extern const {} mp_native_qstr_val_table[];".format(qstr_type), file=f)
-        print("extern const mp_uint_t mp_native_qstr_obj_table[];", file=f)
+        print("extern const uint16_t mp_native_qstr_table[];", file=f)
+        print("extern const mp_uint_t mp_native_obj_table[];", file=f)
 
 
 def do_link(args):
